require "helper"
require "logger"
require "twirl/instrumentation/log_subscriber"

class LogSubscriberInstrumentationTest < Minitest::Test
  def setup
    @cluster = build(:cluster, {
      instrumenter: ActiveSupport::Notifications,
      queues: %w(testing),
    })
    @io = StringIO.new
    logger = Logger.new(@io)
    logger.formatter = proc { |severity, datetime, progname, msg| "#{msg}\n" }
    Twirl::Instrumentation::LogSubscriber.logger = logger
  end

  def log
    @io.string
  end

  def teardown
    Twirl::Instrumentation::LogSubscriber.logger = nil
  end

  def test_set
    @cluster.set "testing", "data"
    line = find_line("Twirl op(set)")
    assert_match %r{queue_name=testing bytes=4}, line
  end

  def test_get
    @cluster.set "testing", "data"
    @cluster.get "testing"
    line = find_line("Twirl op(get)")
    assert_match %r{queue_name=testing bytes=4}, line
  end

  def test_reserve
    @cluster.set "testing", "data"
    @cluster.reserve "testing"
    line = find_line("Twirl op(reserve)")
    assert_match %r{queue_name=testing bytes=4}, line
  end

  def find_line(str)
    regex = /#{Regexp.escape(str)}/
    lines = log.split("\n")
    lines.detect { |line| line =~ regex } ||
      raise("Could not find line matching #{str.inspect} in #{lines.inspect}")
  end
end
